// -*- C++ -*-
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Written (W) 2010 Christian Leistner, leisti@gmail.com
 * Copyright (C) 2010 Christian Leistner, Amir Saffari, Samuel Schulter
 *                    Institute for Computer Graphics and Vision,
 *                    Graz University of Technology, Austria
 */

#include "Bag.h"
#include "Sample.h"

Bag::Bag(int label, double weight, MatrixXd sampleData, VectorXi sampleLabels, VectorXd sampleWeights,
		VectorXi global_sample_index)
{
	this->bag_label_ = label;
	int num_samples = sampleData.rows();
	int num_features = sampleData.cols();
	double sampleWeight = 0.0;
	int sampleLabel = 0;
	VectorXd data;
	this->samples_.resize(num_samples);
	for (int s = 0; s < num_samples; s++)
	{
		sampleLabel = sampleLabels(s);
		sampleWeight = sampleWeights(s);
		data = VectorXd(num_features);
		for (int f = 0; f < num_features; f++)
			data(f) = sampleData(s, f);
		this->samples_[s] = new Sample(data, label, sampleLabel, weight, global_sample_index(s));
	}
}

Bag::~Bag()
{
	if ((int)this->samples_.size() > 0)
	{
		for (int i_sample = 0; i_sample < (int)this->samples_.size(); i_sample++)
		{
			delete(this->samples_[i_sample]);
		}
	}
}
