// -*- C++ -*-
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Written (W) 2010 Christian Leistner, leisti@gmail.com
 * Copyright (C) 2010 Christian Leistner, Amir Saffari, Samuel Schulter
 *                    Institute for Computer Graphics and Vision,
 *                    Graz University of Technology, Austria
 */

#include "FileTools.h"

void FileTools::create()
{
	ofstream file;
	file.open(m_fileName.c_str(), ios::trunc | ios::binary);
	checkAccess(file);
	file << "File created and truncated ..." << endl;
	file.close();
}

void FileTools::read(MatrixXd& data) {
    ifstream file(m_fileName.c_str(), ios::binary);
    checkAccess(file);
    readHeader(file);

    data = MatrixXd(m_numRows, m_numCols);
    for (int nRow = 0; nRow < m_numRows; nRow++) {
        for (int nCol = 0; nCol < m_numCols; nCol++) {
            file >> data(nRow, nCol);
        }
    }

    file.close();
}

void FileTools::read(VectorXd& data) {
    ifstream file(m_fileName.c_str(), ios::binary);
    checkAccess(file);
    readHeader(file);

    data = VectorXd(m_numRows);
    for (int nRow = 0; nRow < m_numRows; nRow++) {
        file >> data[nRow];
    }

    file.close();
}

void FileTools::read(VectorXi& data) {
    ifstream file(m_fileName.c_str(), ios::binary);
    checkAccess(file);
    readHeader(file);

    data = VectorXi(m_numRows);
    for (int nRow = 0; nRow < m_numRows; nRow++) {
        file >> data[nRow];
    }

    file.close();
}

void FileTools::write(const MatrixXd& data) {
    m_numRows = data.rows();
    m_numCols = data.cols();

    ofstream file(m_fileName.c_str(), ios::binary);
    checkAccess(file);
    writeHeader(file, "double");

    for (int nRow = 0; nRow < m_numRows; nRow++) {
        for (int nCol = 0; nCol < m_numCols; nCol++) {
            file << data(nRow, nCol) << " ";
        }
        file << endl;
    }

    file.close();
}

void FileTools::write(const VectorXd& data) {
    m_numRows = data.size();
    m_numCols = 1;

    ofstream file(m_fileName.c_str(), ios::binary);
    checkAccess(file);
    writeHeader(file, "double");

    for (int nRow = 0; nRow < m_numRows; nRow++) {
        file << data(nRow) << " " << endl;
    }

    file.close();
}

void FileTools::write(const VectorXi& data) {
    m_numRows = data.size();
    m_numCols = 1;

    ofstream file(m_fileName.c_str(), ios::binary);
    checkAccess(file);
    writeHeader(file, "int");

    for (int nRow = 0; nRow < m_numRows; nRow++) {
        file << data(nRow) << " " << endl;
    }

    file.close();
}
