// -*- C++ -*-
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Written (W) 2010 Christian Leistner, leisti@gmail.com
 * Copyright (C) 2010 Christian Leistner, Amir Saffari, Samuel Schulter
 *                    Institute for Computer Graphics and Vision,
 *                    Graz University of Technology, Austria
 */

#ifndef FILETOOLS_H_
#define FILETOOLS_H_

#include <string>
#include <fstream>
#include <vector>
#include <string>
#include <Eigen/Core>
#include "ApplicationParameters.h"

using namespace std;
USING_PART_OF_NAMESPACE_EIGEN


class FileTools
{
	public:
		FileTools() { }
		FileTools(const string& fileName) :
			m_fileName(fileName) {
		}

		void setFileName(const string& fileName) {
			m_fileName = fileName;
		}

		void create();

		void read(MatrixXd& data);
		void read(VectorXd& data);
		void read(VectorXi& data);

		void write(const MatrixXd& data);
		void write(const VectorXd& data);
		void write(const VectorXi& data);

		inline const int getNumClasses() const { return m_numClasses; }

	private:
		string m_fileName;
		int m_numRows;
		int m_numCols;
		int m_numClasses;

		void checkAccess(const ifstream &file) {
			if (!file) {
				cout << "Could not access " << m_fileName << endl;
				exit(EXIT_FAILURE);
			}
		}
		void checkAccess(const ofstream &file) {
			if (!file) {
				cout << "Could not access " << m_fileName << endl;
				exit(EXIT_FAILURE);
			}
		}

		void readHeader(ifstream &file) {
			char tempChar[100];

			file >> tempChar; // Double/Int
			file >> m_numRows;
			file >> m_numCols;
			file >> m_numClasses;
			file >> tempChar; // Dense/Sparse
		}
		void writeHeader(ofstream &file, const string& dataType) {
			file << dataType << endl;
			file << m_numRows << " ";
			file << m_numCols << " ";
			file << m_numClasses << endl;
			file << "dense" << endl; // Dense/Sparse
		}
};

#endif /* FILETOOLS_H_ */
