// -*- C++ -*-
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Written (W) 2010 Christian Leistner, leisti@gmail.com
 * Copyright (C) 2010 Christian Leistner, Amir Saffari, Samuel Schulter
 *                    Institute for Computer Graphics and Vision,
 *                    Graz University of Technology, Austria
 */

#ifndef MILFOREST_H_
#define MILFOREST_H_

#include "ApplicationParameters.h"
#include "MILDataset.h"
#include "RandomForest.h"
#include <vector>
#include "RandomForestResult.h"
#include "math.h"
#include "Utilities.h"


class MILForest
{
private:
	int num_features_;
	int num_classes_;
	RandomForest rf_;

	// Approx implementation
	void trainApprox( MILDataset& data );
	bool doTrainTree( int epoche_value );
	bool doSwitchLabel( int epoche_value );
	double temperatureProbability( int epoche_value, double cooling_parameter );

	// Exact implementation
	void trainExact( MILDataset& data );
	double temperature( int epoche_value, double cooling_parameter, double start_value );
	void estimateLabelProbabilityDistribution( MILDataset& data, double current_temperature );
	void diceLabelsForPositiveBags( MILDataset& data );
	double calculateLoss( double confidence, int label );

public:
	MILForest( int num_features, int num_classes );
	virtual ~MILForest();

	RandomForestResult train( MILDataset& data );
	double trainAndReturnError( MILDataset& data );
	RandomForestResult test( MILDataset& data );
	double testAndReturnError( MILDataset& data );
};

#endif /* MILFOREST_H_ */
