// -*- C++ -*-
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Written (W) 2010 Christian Leistner, leisti@gmail.com
 * Copyright (C) 2010 Christian Leistner, Amir Saffari, Samuel Schulter
 *                    Institute for Computer Graphics and Vision,
 *                    Graz University of Technology, Austria
 */

#include <iostream>
#include "MILForest.h"
#include "MILDataset.h"
#include "ApplicationParameters.h"
#include <string>

using namespace std;


void printHelp()
{
	cout << "--------- HELP -----------------" << endl;
	cout << "Basic usage:" << endl;
	cout << "./MC_MILForest config_file" << endl;
	cout << "That's it, everything else can be configured in the config file" << endl << endl;
}

int main(int argc, char** argv)
{
	cout << endl << "################### MILForest ##################" << endl << endl;


	// parse the input parameters
	if ( argc == 1 )
	{
		cout << "No input arguments specified: aborting." << endl;
		printHelp();
		exit(-1);
	}
	string config_file_path = argv[1];


	// initialize the application parameters
	cout << "========== Application parameters ==========" << endl;
	ApplicationParameters::config_file_name =  config_file_path;
	ApplicationParameters::instance(); // just to get a reasonable output if the debug flag is on


	// load the dataset
	cout << endl << "========== Dataset ==========" << endl;
	MILDataset trainset;
	MILDataset testset;
	{
		MILDataset total_dataset;
		total_dataset.readData( ApplicationParameters::instance()->getDatafile(),
				ApplicationParameters::instance()->getSamplelabelsFile(),
				ApplicationParameters::instance()->getBagsToSamplesFile(),
				ApplicationParameters::instance()->getBaglabelsFile());
		total_dataset.splitDataset(trainset, testset, ApplicationParameters::instance()->getTrainingSetFile(),
				ApplicationParameters::instance()->getTestSetFile() );
	}
	if ( ApplicationParameters::instance()->isDebugOn() )
	{
		cout << "Data file: " << ApplicationParameters::instance()->getDatafile() << endl;
		cout << "Trainset has " << trainset.getNumBags() << " bags and "
				<< trainset.getNumSamples() << " samples" << endl;
		cout << "Testset has " << testset.getNumBags() << " bags and "
				<< testset.getNumSamples() << " samples" << endl;
	}


	// Train and test a Random Forest
	cout << endl << "========== Training & Testing ==========" << endl;
	MILForest milf = MILForest( trainset.getNumFeatures(), trainset.getNumClasses() );
	RandomForestResult train_result = milf.train( trainset );
	RandomForestResult test_result = milf.test( testset );


	// Display the result
	cout << endl << "========== Result ==========" << endl;
	cout << "Sample classifier: " << endl;
	cout << "\tTraining set: ";
	train_result.calculateError( trainset.getSamples(), true );
	cout << "\tTest set: ";
	test_result.calculateError( testset.getSamples(), true );

	cout << "Bag classifier:" << endl;
	cout << "\tTraining set: ";
	train_result.calculateError( trainset.getBags(), true );
	cout << "\tTest set: ";
	test_result.calculateError( testset.getBags(), true );


	return 0;
}
