// -*- C++ -*-
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Written (W) 2010 Christian Leistner, leisti@gmail.com
 * Copyright (C) 2010 Christian Leistner, Amir Saffari, Samuel Schulter
 *                    Institute for Computer Graphics and Vision,
 *                    Graz University of Technology, Austria
 */

#ifndef RANDOMFOREST_H_
#define RANDOMFOREST_H_

#include "ApplicationParameters.h"
#include "RandomTree.h"
#include "Sample.h"
#include <vector>
#include "RandomNodeConfidence.h"
#include "RandomForestResult.h"

using namespace std;


class RandomForest {
private:
	int num_trees_;
	int num_features_;
	int num_classes_;
	vector<RandomTree*> trees_;

	void subsampleWithReplacement( const vector<Sample*> samples, vector<Sample*>& inbag_samples, vector<Sample*>& outbag_samples );
	void permuteSamples( const vector<Sample*> samples, vector<Sample*>& permuted_samples );
	void trainTreeWithBagging( int tree_index, const vector<Sample*> samples );
	void trainTreeWithoutBagging( int tree_index, const vector<Sample*> samples );
	void trainTreeWithBaggingAndRefine( int tree_index, const vector<Sample*> samples );
	void trainTreeWithPermutation( int tree_index, const vector<Sample*> samples );

public:
	RandomForest( int num_features, int num_classes );
	virtual ~RandomForest();

	double trainAndReturnError( const vector<Sample*> samples );
	RandomForestResult train( const vector<Sample*> samples );
	double testAndReturnError( const vector<Sample*> samples, bool show_result = false );
	RandomForestResult test( const vector<Sample*> samples );
	void trainTree( int tree_index, const vector<Sample*> samples );
	void clearTree( int tree_index );
};

#endif /* RANDOMFOREST_H_ */
