// -*- C++ -*-
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Written (W) 2010 Christian Leistner, leisti@gmail.com
 * Copyright (C) 2010 Christian Leistner, Amir Saffari, Samuel Schulter
 *                    Institute for Computer Graphics and Vision,
 *                    Graz University of Technology, Austria
 */

#include "RandomForestResult.h"

RandomForestResult::RandomForestResult()
{
}

RandomForestResult::RandomForestResult( MatrixXd conf )
{
	this->confidences_ = conf;
	int num_samples = conf.rows();
	VectorXi tmp_pred = VectorXi::Zero( num_samples );
	for ( int s = 0; s < num_samples; s++ )
	{
		conf.row(s).maxCoeff( &tmp_pred(s) );
	}
	this->predictions_ = tmp_pred;
}

RandomForestResult::RandomForestResult( MatrixXd conf, VectorXi pred )
{
	this->confidences_ = conf;
	this->predictions_ = pred;
}

RandomForestResult::~RandomForestResult()
{
}

double RandomForestResult::calculateError( const vector< Sample* > samples, bool show_result )
{
	int num_wrong_predictions = 0;
	int num_samples = (int)samples.size();
	for ( int s = 0; s < num_samples; s++ )
	{
		if ( samples[s]->getCorrectLabel() != predictions_(s) )
			num_wrong_predictions++;
	}

	double error = (double)num_wrong_predictions / (double)num_samples;

	if (show_result)
	{
		cout << "Wrong sample predictions: " << num_wrong_predictions << " / " << num_samples << " -> error = " << error << endl;
	}

	return error;
}

double RandomForestResult::calculateError( const vector<Bag*> bags, bool show_result )
{
	int global_sample_index = -1;

	int num_bags = (int)bags.size();
	int num_wrong_bag_predictions = 0;

	// iterate all bags
	for ( int b = 0; b < num_bags; b++ )
	{
		// find the max confidence for the current bag
		int max_class = 0;
		int tmp_class = -1;
		double tmp_conf = 0.0;
		for ( int s = 0; s < bags[b]->getNumSamples(); s++ )
		{
			global_sample_index = bags[b]->getSamples()[s]->getGlobalSampleIndex();
			tmp_conf = confidences_.row( global_sample_index ).maxCoeff( &tmp_class );
			if ( tmp_class == 1)
				max_class = 1;
		}

		// check prediction
		if ( max_class != bags[b]->getBagLabel() )
			num_wrong_bag_predictions++;
	}

	double error = (double)num_wrong_bag_predictions / (double)num_bags;

	if (show_result)
	{
		cout << "Wrong bag predictions: " << num_wrong_bag_predictions << " / " << num_bags << " -> error = " << error << endl;
	}

	return error;
}
