// -*- C++ -*-
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Written (W) 2010 Christian Leistner, leisti@gmail.com
 * Copyright (C) 2010 Christian Leistner, Amir Saffari, Samuel Schulter
 *                    Institute for Computer Graphics and Vision,
 *                    Graz University of Technology, Austria
 */

#ifndef RANDOMFORESTRESULT_H_
#define RANDOMFORESTRESULT_H_

#include <Eigen/Core>
#include "Sample.h"
#include "Bag.h"
#include <vector>

USING_PART_OF_NAMESPACE_EIGEN

class RandomForestResult
{
private:
	MatrixXd confidences_;
	VectorXi predictions_;

public:
	RandomForestResult();
	RandomForestResult( MatrixXd conf );
	RandomForestResult( MatrixXd conf, VectorXi pred );
	virtual ~RandomForestResult();

	inline MatrixXd getConfidences() const { return this->confidences_; }
	inline VectorXi getPredictions() const { return this->predictions_; }

	double calculateError( const vector<Sample*> samples, bool show_result = false );
	double calculateError( const vector<Bag*> bags, bool show_result = false );
};

#endif /* RANDOMFORESTRESULT_H_ */
