// -*- C++ -*-
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Written (W) 2010 Christian Leistner, leisti@gmail.com
 * Copyright (C) 2010 Christian Leistner, Amir Saffari, Samuel Schulter
 *                    Institute for Computer Graphics and Vision,
 *                    Graz University of Technology, Austria
 */

#ifndef RANDOMNODE_H_
#define RANDOMNODE_H_

#include "ApplicationParameters.h"
#include "Sample.h"
#include "RandomNodeConfidence.h"
#include <vector>
#include <Eigen/Core>

USING_PART_OF_NAMESPACE_EIGEN


class RandomNode {
private:
	bool is_leaf_;
	int depth_;
	int num_classes_;
	int num_features_;

	// Split node members
	VectorXi random_feature_indices_;
	VectorXd random_feature_weights_;
	double threshold_;
	RandomNode* leftChild_;
	RandomNode* rightChild_;

	// Leaf node members
	RandomNodeConfidence nodeConf_;

	// Methods
	bool doSplit( const vector<Sample*> samples );
	void findSplittingHypothesis( const vector<Sample*> samples );
	std::pair<double, double> calculateGiniAndThreshold( const vector<Sample*> samples, vector< std::pair<double, int> > responses );
	void splitNode( const vector<Sample*> samples, vector<Sample*>& leftNodeSamples, vector<Sample*>& rightNodeSamples );

public:
	RandomNode( int depth, int num_features, int num_classes );
	virtual ~RandomNode();
	void train( const vector<Sample*> samples );
	RandomNodeConfidence test( const Sample* sample );
	void refine( const vector<Sample*> sampels );
};

#endif /* RANDOMNODE_H_ */
