// -*- C++ -*-
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Written (W) 2010 Christian Leistner, leisti@gmail.com
 * Copyright (C) 2010 Christian Leistner, Amir Saffari, Samuel Schulter
 *                    Institute for Computer Graphics and Vision,
 *                    Graz University of Technology, Austria
 */

#include "RandomNodeConfidence.h"


RandomNodeConfidence::RandomNodeConfidence( int num_classes )
{
	this->class_confidence_ = VectorXd::Zero( num_classes );
	this->node_label_ = 0;
}

RandomNodeConfidence::~RandomNodeConfidence()
{
}

void RandomNodeConfidence::calculateConfidence( const vector<Sample*> samples )
{
	int num_samples = (int)samples.size();
	int label_idx;

	if (num_samples > 0)
	{
		// Sum up all weights for the according class/label ... additionally sum up all weights for normalization
		double total_weight = 0.0;

		for ( int s = 0; s < num_samples; s++ )
		{
			//labelIdx = (samples[s]->getLabel() == 0) ? 0 : 1; // binary class
			label_idx = samples[s]->getLabel();
			class_confidence_(label_idx) += samples[s]->getWeight();
			total_weight += samples[s]->getWeight();
		}
		normalzing_weight_ = total_weight;

		// normalize the confidences and find the best confidence, i.e. the best class
		int num_classes = class_confidence_.size();

		class_confidence_(0) /= normalzing_weight_;
		node_label_ = 0;
		for ( int c = 1; c < num_classes; c++ )
		{
			class_confidence_(c) /= normalzing_weight_;
		}
		class_confidence_.maxCoeff(&node_label_);
	}
	else
	{
		class_confidence_(0) = 1.0;
		node_label_ = 0;
	}
}

void RandomNodeConfidence::refineConfidence( const vector<Sample*> samples )
{
	int num_samples = (int)samples.size();
	VectorXd new_conf = VectorXd::Zero( class_confidence_.size() );

	double new_total_weight = 0.0;
	int label_index;
	for ( int s = 0; s < num_samples; s++ )
	{
		//label_index = (samples[s]->getLabel() == 0) ? 0 : 1; // binary class
		label_index = samples[s]->getLabel();
		new_conf(label_index) += samples[s]->getWeight();
		new_total_weight += samples[s]->getWeight();
	}

	class_confidence_ = (class_confidence_ * normalzing_weight_ + new_conf) / (normalzing_weight_ + new_total_weight);
	normalzing_weight_ += new_total_weight;
	class_confidence_.maxCoeff(&node_label_);
}

double RandomNodeConfidence::getClassConfidence( int class_index )
{
	return class_confidence_( class_index );
}
