// -*- C++ -*-
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Written (W) 2010 Christian Leistner, leisti@gmail.com
 * Copyright (C) 2010 Christian Leistner, Amir Saffari, Samuel Schulter
 *                    Institute for Computer Graphics and Vision,
 *                    Graz University of Technology, Austria
 */

#include "RandomTree.h"

RandomTree::RandomTree( int num_features, int num_classes ) : num_classes_(num_classes)
{
	rootNode_ = new RandomNode( 0, num_features, num_classes ); // depth = 0
}

RandomTree::~RandomTree()
{
	delete( rootNode_ );
}

void RandomTree::train( const vector<Sample*> samples )
{
	rootNode_->train( samples );
}

RandomNodeConfidence RandomTree::test( const Sample* sample )
{
	return rootNode_->test( sample );
}

RandomForestResult RandomTree::test( const vector<Sample*> samples )
{
	RandomNodeConfidence sample_confidence;
	int num_samples = (int)samples.size();
	MatrixXd tree_confidences = MatrixXd::Zero(num_samples, num_classes_);
	VectorXi tree_predictions = VectorXi::Zero(num_samples);
	for ( int s = 0; s < num_samples; s++ )
	{
		sample_confidence = rootNode_->test(samples[s]);
		tree_confidences.row(s) = sample_confidence.getClassConfidences();
		sample_confidence.getClassConfidences().maxCoeff(&tree_predictions(s));
	}

	return RandomForestResult( tree_confidences, tree_predictions );
}

void RandomTree::refine( const vector<Sample*> samples )
{
	rootNode_->refine(samples);
}
