// -*- C++ -*-
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Written (W) 2010 Christian Leistner, leisti@gmail.com
 * Copyright (C) 2010 Christian Leistner, Amir Saffari, Samuel Schulter
 *                    Institute for Computer Graphics and Vision,
 *                    Graz University of Technology, Austria
 */

#ifndef SAMPLE_H_
#define SAMPLE_H_

#include <Eigen/Core>

using namespace std;
USING_PART_OF_NAMESPACE_EIGEN

class Sample
{
	private:
		int label_;
		int correct_label_;
		double weight_;
		VectorXd features_;
		int global_index_;
		VectorXd label_probabilities_;

	public:
		Sample(VectorXd features, int label, double weight, int global_index = -1);
		Sample( VectorXd features, int label, int correct_label, double weight, int global_index = -1 );
		virtual ~Sample();

		// Getters
		VectorXd getFeatures() const { return this->features_; }
		VectorXd getFeatures(VectorXi feature_indices) const;
		int getNumFeatures() const { return this->features_.size(); }
		int getLabel() const { return this->label_; }
		int getCorrectLabel() const { return this->correct_label_; }
		double getWeight() const { return this->weight_; }
		inline int getGlobalSampleIndex() const { return this->global_index_; }
		inline void setGlobalSampleIndex( int global_index ) { this->global_index_ = global_index; }
		VectorXd getLabelProbabilities() const { return this->label_probabilities_; }

		// Setters
		void setLabel(int new_label) { this->label_ = new_label; }
		void setWeihgt(double new_weight) { this->weight_ = new_weight; }
		void setLabelProbabilities(VectorXd new_probabilities) { this->label_probabilities_ = new_probabilities; }


		void diceNewLabel();
};

#endif /* SAMPLE_H_ */
