// -*- C++ -*-
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Written (W) 2010 Christian Leistner, leisti@gmail.com
 * Copyright (C) 2010 Christian Leistner, Amir Saffari, Samuel Schulter
 *                    Institute for Computer Graphics and Vision,
 *                    Graz University of Technology, Austria
 */

#include <algorithm>
#include <set>
#include <time.h>
#include "Utilities.h"

VectorXi randPerm(const int& num)
{
    VectorXi outRand(num);
    vector<pair<double, int> > pairs;
    pair<double, int> tmpPair;

    for (int n = 0; n < num; n++)
    {
        tmpPair.first = randDouble();
        tmpPair.second = n;
        pairs.push_back(tmpPair);
    }
    sort(pairs.begin(), pairs.end());

    for (int n = 0; n < num; n++)
    {
        outRand(n) = pairs[n].second;
    }

    return outRand;
}

VectorXi randPerm(const int& num, const int& numPart)
{
    VectorXi outRand(numPart);
    vector<pair<double, int> > pairs;
    pair<double, int> tmpPair;

    for (int n = 0; n < num; n++)
    {
        tmpPair.first = randDouble();
        tmpPair.second = n;
        pairs.push_back(tmpPair);
    }
    sort(pairs.begin(), pairs.end());

    for (int n = 0; n < numPart; n++)
    {
        outRand(n) = pairs[n].second;
    }

    return outRand;
}

//! Returns the time (ms) elapsed between two calls to this function
double timeIt(int reset)
{
    static time_t startTime, endTime;
    static int timerWorking = 0;

    if (reset)
    {
        startTime = time(NULL);
        timerWorking = 1;
        return -1;
    }
    else
    {
        if (timerWorking)
        {
            endTime = time(NULL);
            timerWorking = 0;
            return (double) (endTime - startTime);
        }
        else
        {
            startTime = time(NULL);
            timerWorking = 1;
            return -1;
        }
    }
}

void fillWithRandomNumbers(std::vector<double>& tmpWeights)
{

  std::vector<double>::iterator it(tmpWeights.begin()), end(tmpWeights.end());
  for(; it != end;it++){
     *it = 2.0*(randDouble() - 0.5);
  }
}
void fillWithRandomNumbers(VectorXd& num_vector)
{
	int num_values = num_vector.size();
	for (int v = 0; v < num_values; v++)
		num_vector(v) = 2.0 * (randDouble() - 0.5);
}
