/*
 * Utilities.h
 *
 *  Created on: Aug 13, 2009
 *      Author: amir
 */

#ifndef UTILITIES_H_
#define UTILITIES_H_

#include <string>
#include <math.h>
#include <time.h>
#include <fstream>
#include <sys/time.h>
#include <random/uniform.h>
#include <random/normal.h>
#include <Eigen/Core>

//#include "Data.h"

using namespace std;
using namespace ranlib;
USING_PART_OF_NAMESPACE_EIGEN


inline unsigned int getDevRandom()  {
    ifstream devFile("/dev/urandom", ios::binary);
    unsigned int outInt = 0;
    char tempChar[sizeof(outInt)];

    devFile.read(tempChar, sizeof(outInt));
    outInt = atoi(tempChar);

    devFile.close();

    return outInt;
}

inline double randDouble() {
    static int seedCount = 0;
    static Uniform<double> uniformRandGen;

    if (!seedCount) {
        struct timeval TV;
        unsigned int curTime;

        gettimeofday(&TV, NULL);
        curTime = (unsigned int) TV.tv_usec;
        uniformRandGen.seed((unsigned int) time(NULL) + curTime + getpid() + getDevRandom());

        seedCount = 1;
    }

    return uniformRandGen.random();
}

inline int randInteger(const int& minRange, const int& maxRange) {
    return (int) (minRange + (maxRange - minRange) * randDouble());
}

inline VectorXi randInteger(const int& minRange, const int& maxRange, const int& numSample) {
    VectorXi out(numSample);
    int range = maxRange - minRange;
    for (int nSamp = 0; nSamp < numSample; nSamp++) {
        out(nSamp) = minRange + range * randDouble();
    }
    return out;
}

VectorXi randPerm(const int& num);

VectorXi randPerm(const int& num, const int& numPart);

//! Returns the time (ms) elapsed between two calls to this function
double timeIt(int reset);

void fillWithRandomNumbers(std::vector<double>& tmpWeights);

void fillWithRandomNumbers(VectorXd& num_vector);

#endif /* UTILITIES_H_ */
